@@ -278,6 +278,16 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
       return NO_ERR;
     }
 
+    // strcasecmp shall be used, but it'll break gmail gadget.
+    // Microsoft XHR is also case sensitive.
+    if (strcmp(header, "Cookie") == 0 &&
+        value && strcasecmp(value, "none") == 0) {
+      // Microsoft XHR hidden feature: setRequestHeader('Cookie', 'none')
+      // clears all cookies. Some gadgets (e.g. reader) use this.
+      curl_easy_setopt(curl_, CURLOPT_COOKIELIST, "ALL");
+      return NO_ERR;
+    }
+
     std::string header_str(header);
     CaseInsensitiveStringMap::iterator it =
         request_headers_map_.find(header_str);
@@ -423,6 +433,11 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
   }
 
   virtual ExceptionCode Send(const DOMDocumentInterface *data) {
+    if (request_headers_map_.find("Content-Type") ==
+        request_headers_map_.end()) {
+      // Set content type if it's not set yet.
+      request_headers_map_["Content-Type"] = "application/xml;charset=UTF-8";
+    }
     return Send(data ? data->GetXML() : std::string());
   }
 
@@ -483,7 +498,8 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
 
     data_size = std::min(data_size, context->request_data.size() -
                                     context->request_offset);
-    memcpy(ptr, context->request_data.c_str(), data_size);
+    memcpy(ptr, context->request_data.c_str() + context->request_offset,
+           data_size);
     context->request_offset += data_size;
     return data_size;
   }
@@ -676,7 +692,7 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
 
     // Streamed mode.
     if (ondatareceived_signal_.HasActiveConnections())
-      return ondatareceived_signal_(data);
+      return ondatareceived_signal_(data.c_str(), size);
 
     // Normal mode.
     if (CheckSize(response_body_.length(), size, 1)) {
@@ -892,7 +908,7 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
   }
 
   virtual Connection *ConnectOnDataReceived(
-      Slot1<size_t, const std::string &> *receiver) {
+      Slot2<size_t, const void *, size_t> *receiver) {
     return ondatareceived_signal_.Connect(receiver);
   }
 
@@ -990,7 +1006,7 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
   CaseInsensitiveStringMap request_headers_map_;
   CaseInsensitiveStringMap response_headers_map_;
   Signal0<void> onreadystatechange_signal_;
-  Signal1<size_t, const std::string &> ondatareceived_signal_;
+  Signal2<size_t, const void *, size_t> ondatareceived_signal_;
 
   std::string url_;
   std::string host_;
@@ -1100,7 +1116,7 @@ class XMLHttpRequestFactory : public XMLHttpRequestFactoryInterface {
     CURL *share_ref;
   };
 
-  typedef std::map<int, Session> Sessions;
+  typedef LightMap<int, Session> Sessions;
   Sessions sessions_;
   int next_session_id_;
   std::string default_user_agent_;
