@@ -77,9 +77,12 @@
 
 #include <ggadget/common.h>
 #include <ggadget/digest_utils.h>
+#include <ggadget/light_map.h>
 #include <ggadget/string_utils.h>
 #include <ggadget/system_utils.h>
 
+using ggadget::LightMap;
+
 #define NO_NAMESPACE
 #include "browser_child.h"
 #include "browser_child_internal.h"
@@ -105,22 +108,26 @@ struct BrowserObjectInfo {
   JSObject *js_object;
 };
 
-typedef std::map<size_t, BrowserObjectInfo> BrowserObjectMap;
+typedef LightMap<size_t, BrowserObjectInfo> BrowserObjectMap;
 class HostObjectWrapper;
-typedef std::map<size_t, HostObjectWrapper *> HostObjectMap;
+typedef LightMap<size_t, HostObjectWrapper *> HostObjectMap;
 
 struct BrowserInfo {
-  BrowserInfo() : embed(NULL), browser_id(0), check_load_timer(0) { }
+  BrowserInfo()
+      : embed(NULL), browser_id(0), check_load_timer(0),
+        always_open_new_window(true) {
+  }
   GtkMozEmbed *embed;
   size_t browser_id;
   BrowserObjectMap browser_objects;
   HostObjectMap host_objects;
   int check_load_timer;
+  bool always_open_new_window;
 };
 
 static size_t g_browser_object_seq = 0;
 
-typedef std::map<size_t, BrowserInfo> BrowserMap;
+typedef LightMap<size_t, BrowserInfo> BrowserMap;
 static BrowserMap g_browsers;
 
 // The singleton GtkMozEmbed instance for temporary use when a new window
@@ -579,7 +586,7 @@ class ContentPolicy : public nsIContentPolicy {
                         PRInt16 *retval) {
     NS_ENSURE_ARG_POINTER(content_location);
     NS_ENSURE_ARG_POINTER(retval);
-    nsCString url_spec, origin_spec, url_scheme;
+    nsCString url_spec, origin_spec;
     content_location->GetSpec(url_spec);
     if (content_type == TYPE_DOCUMENT && g_embed_for_new_window) {
       // Handle a new window request.
@@ -593,44 +600,61 @@ class ContentPolicy : public nsIContentPolicy {
           break;
         }
       }
-      // Reject this URL no matter if the controler has opened it.
+      // Reject this URL no matter if the controller has opened it.
       *retval = REJECT_OTHER;
       return NS_OK;
     }
 
     *retval = ACCEPT;
-    content_location->GetScheme(url_scheme);
-    if (content_type == TYPE_DOCUMENT || content_type == TYPE_SUBDOCUMENT) {
-      // If the URL is opened the first time in a blank window or frame,
-      // request_origin is NULL or "about:blank".
-      if (request_origin) {
-        request_origin->GetSpec(origin_spec);
-        SendLog("ShouldLoad:\n"
-                " origin: %s\n"
-                "    url: %s", origin_spec.get(), url_spec.get());
+    PRBool is_loading = PR_FALSE;
+    BrowserInfo *browser_info = FindBrowserByContentPolicyContext(
+        context, &is_loading);
+    if (!browser_info || !browser_info->always_open_new_window)
+      return NS_OK;
 
-        // Treats urls with the same base url but different refs as equal.
-        std::string tmp_origin(origin_spec.get());
-        std::string tmp_url(url_spec.get());
-        tmp_origin = tmp_origin.substr(0, tmp_origin.find_last_of('#'));
-        tmp_url = tmp_url.substr(0, tmp_url.find_last_of('#'));
-        if (tmp_origin != tmp_url &&
-            !origin_spec.Equals(nsCString("about:blank")) &&
-            !url_scheme.Equals(nsCString("javascript"))) {
-          PRBool is_loading = PR_FALSE;
-          BrowserInfo *browser_info = FindBrowserByContentPolicyContext(
-              context, &is_loading);
-          // Allow URLs opened during page loading to be opened in place.
-          if (browser_info && !is_loading) {
-            std::string r = SendFeedback(browser_info->browser_id,
-                                         kOpenURLFeedback, url_spec.get(),
-                                         NULL);
-            // The controller should have opened the URL, so don't let the
-            // embedded browser open it.
-            if (r[0] != '0')
-              *retval = REJECT_OTHER;
-          }
+    if (content_type == TYPE_DOCUMENT || content_type == TYPE_SUBDOCUMENT) {
+      if (request_origin)
+        request_origin->GetSpec(origin_spec);
+
+      SendLog("ShouldLoad: is_loading=%d\n"
+              " origin: %s\n"
+              "    url: %s", is_loading, origin_spec.get(), url_spec.get());
+
+      // If the URL is opened the first time in a blank window or frame,
+      // or the URL is dragged and dropped on the browser, request_origin
+      // is NULL or "about:blank".
+      if (!request_origin || origin_spec.Equals(nsCString("about:blank"))) {
+        if (!browser_info->check_load_timer) {
+          // Reject the request that is not initiated by the controller.
+          // It may be initiated by user drag-and-drop.
+          *retval = REJECT_OTHER;
         }
+        // Otherwise let the URL loaded in place.
+        return NS_OK;
+      }
+
+      nsCString url_scheme;
+      content_location->GetScheme(url_scheme);
+      if (url_scheme.Equals(nsCString("javascript"))) {
+        // Also let javascript URLs handled in place.
+        return NS_OK;
+      }
+
+      // Treats urls with the same base url but different refs as equal.
+      std::string tmp_origin(origin_spec.get());
+      std::string tmp_url(url_spec.get());
+      tmp_origin = tmp_origin.substr(0, tmp_origin.find_last_of('#'));
+      tmp_url = tmp_url.substr(0, tmp_url.find_last_of('#'));
+      // Allow URLs opened during page loading to be opened in place, otherwise
+      // call the kOpenURLFeedback to let the controller handle.
+      if (tmp_origin != tmp_url && !is_loading) {
+        std::string r = SendFeedback(browser_info->browser_id,
+                                     kOpenURLFeedback, url_spec.get(),
+                                     NULL);
+        // The controller should have opened the URL, so don't let the
+        // embedded browser open it.
+        if (r[0] != '0')
+          *retval = REJECT_OTHER;
       }
     }
     return NS_OK;
@@ -747,7 +771,7 @@ static gboolean CheckContentLoaded(gpointer data) {
 static void NewBrowser(int param_count, const char **params, size_t id) {
   if (param_count != 3) {
     SendLog("Incorrect param count for %s: 3 expected, %d given.",
-            kSetContentCommand, param_count);
+            kNewBrowserCommand, param_count);
     return;
   }
 
@@ -971,6 +995,21 @@ static std::string CallBrowserObject(BrowserInfo *browser_info,
                                info.object_id);
 }
 
+static void SetAlwaysOpenNewWindow(int param_count, const char **params,
+                                   size_t id) {
+  if (param_count != 3) {
+    SendLog("Incorrect param count for %s: 3 expected, %d given.",
+            kSetAlwaysOpenNewWindowCommand, param_count);
+    return;
+  }
+
+  BrowserInfo *info = GetBrowserInfo(id);
+  if (!info)
+    return;
+
+  info->always_open_new_window = (params[2][0] == '1');
+}
+
 static void ProcessCommand(int param_count, const char **params) {
   std::string result(kReplyPrefix);
   if (strcmp(params[0], kQuitCommand) == 0) {
@@ -997,6 +1036,8 @@ static void ProcessCommand(int param_count, const char **params) {
           OpenURL(param_count, params, id);
         } else if (strcmp(params[0], kCloseBrowserCommand) == 0) {
           RemoveBrowser(id);
+        } else if (strcmp(params[0], kSetAlwaysOpenNewWindowCommand) == 0) {
+          SetAlwaysOpenNewWindow(param_count, params, id);
         } else if (param_count < 3) {
           SendLog("No enough command parameters or invalid command: %s\n",
                   params[0]);
@@ -1173,7 +1214,7 @@ static bool InitGecko() {
 
   static const GREVersionRange kGREVersion = {
     "1.9a", PR_TRUE,
-    "1.9.0.*", PR_TRUE
+    "1.9.2", PR_FALSE,
   };
 
   char xpcom_location[4096];
