@@ -13,8 +13,9 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 */
-#include <map>
+
 #include <string>
+#include <ggadget/light_map.h>
 #include <ggadget/logger.h>
 #include <ggadget/slot.h>
 #include <ggadget/unicode_utils.h>
@@ -33,7 +34,7 @@
 namespace ggadget {
 namespace qt {
 
-static std::map<QScriptEngine*, JSScriptContext*> *g_data = NULL;
+static LightMap<QScriptEngine*, JSScriptContext*> *g_data = NULL;
 
 JSScriptContext *GetEngineContext(QScriptEngine *engine) {
   return (*g_data)[engine];
@@ -44,7 +45,7 @@ static JSScriptContext::Impl* GetEngineContextImpl(QScriptEngine *engine) {
 }
 
 void InitScriptContextData() {
-  if (!g_data) g_data = new std::map<QScriptEngine*, JSScriptContext*>();
+  if (!g_data) g_data = new LightMap<QScriptEngine*, JSScriptContext*>();
 }
 
 // String.substr is not ecma standard and qtscript doesn't provide it, so make
@@ -177,7 +178,7 @@ class JSScriptContext::Impl {
   Impl(JSScriptContext *parent)
       : parent_(parent), resolver_(NULL), line_number_(0) {}
   ~Impl() {
-    std::map<ScriptableInterface*, ResolverScriptClass*>::iterator iter;
+    LightMap<ScriptableInterface*, ResolverScriptClass*>::iterator iter;
     for (iter = script_classes_.begin(); iter != script_classes_.end(); iter++) {
       delete iter->second;
     }
@@ -248,8 +249,8 @@ class JSScriptContext::Impl {
 
   QScriptEngine engine_;
   JSScriptContext *parent_;
-  std::map<std::string, Slot*> class_constructors_;
-  std::map<ScriptableInterface*, ResolverScriptClass*> script_classes_;
+  LightMap<std::string, Slot*> class_constructors_;
+  LightMap<ScriptableInterface*, ResolverScriptClass*> script_classes_;
   Signal1<void, const char *> error_reporter_signal_;
   Signal2<bool, const char *, int> script_blocked_signal_;
   ResolverScriptClass *resolver_;
@@ -509,15 +510,16 @@ void ResolverScriptClass::setProperty(QScriptValue &object,
     DLOG("setPropertyByIndex:%s=%s", sname.c_str(), val.Print().c_str());
   } else {
     Variant proto;
-    if (object_->GetPropertyInfo(sname.c_str(), &proto)
-        == ScriptableInterface::PROPERTY_DYNAMIC) {
-      ConvertJSToNativeVariant(engine(), value, &val);
-    } else {
+    ScriptableInterface::PropertyType prop_type =
+        ScriptableInterface::PROPERTY_NOT_EXIST;
+    prop_type = object_->GetPropertyInfo(sname.c_str(), &proto);
+    if (prop_type == ScriptableInterface::PROPERTY_NORMAL ||
+        prop_type == ScriptableInterface::PROPERTY_DYNAMIC) {
       ConvertJSToNative(engine(), proto, value, &val);
+      DLOG("setProperty:proto:%s", proto.Print().c_str());
+      object_->SetProperty(sname.c_str(), val);
+      DLOG("setProperty:%s=%s", sname.c_str(), val.Print().c_str());
     }
-    DLOG("setProperty:proto:%s", proto.Print().c_str());
-    object_->SetProperty(sname.c_str(), val);
-    DLOG("setProperty:%s=%s", sname.c_str(), val.Print().c_str());
   }
   CheckException(engine()->currentContext(), object_, NULL);
 }
