@@ -22,12 +22,13 @@
 #include <QtCore/QTextStream>
 #endif
 #include <QtCore/QUrl>
-#include <QtCore/QBuffer>
-#include <QtNetwork/QHttp>
+#include <QtCore/QRegExp>
+#include <QtNetwork/QNetworkProxy>
 #include <QtNetwork/QHttpHeader>
 
 #if QT_VERSION >= 0x040400
 #include <QtNetwork/QNetworkCookie>
+#include <QtNetwork/QNetworkCookieJar>
 #endif
 
 #include <ggadget/gadget_consts.h>
@@ -58,49 +59,54 @@ static const Variant kOpenDefaultArgs[] = {
 
 static const Variant kSendDefaultArgs[] = { Variant("") };
 
-class Session {
- public:
 #if QT_VERSION >= 0x040400
-  void RestoreCookie(QHttpRequestHeader *header) {
-    QString str;
-    for (int i = 0; i < cookies_.size(); i++) {
-      str += cookies_[i].toRawForm(QNetworkCookie::NameAndValueOnly);
-      if (i < cookies_.size() - 1) str += "; ";
-    }
-    if (!str.isEmpty()) header->setValue("Cookie", str);
-    DLOG("Cookie:%s", str.toStdString().c_str());
-  }
-  void SaveCookie(const QHttpResponseHeader& header) {
-    QStringList list = header.allValues("Set-Cookie");
-    if (list.size() != 0) DLOG("Get Cookie Line: %d", list.size());
-    for (int i = 0; i < list.size(); i++) {
-      QList<QNetworkCookie> cookies = QNetworkCookie::parseCookies(list[i].toAscii());
-      cookies_ += cookies;
-    }
-  }
-  QList<QNetworkCookie> cookies_;
-#else
-  void RestoreCookie(QHttpRequestHeader *header) {}
-  void SaveCookie(const QHttpResponseHeader& header) {}
-#endif
-};
+// Cookie support is incomplete. Cookies are not persistent after application
+// exits.
+static QNetworkCookieJar* cookie_jar = NULL;
 
-class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
+static void RestoreCookie(const QUrl& url, QHttpRequestHeader *header) {
+  QList<QNetworkCookie> list = cookie_jar->cookiesForUrl(url);
+  QStringList value;
+  foreach (QNetworkCookie cookie, list) {
+    value << cookie.toRawForm(QNetworkCookie::NameAndValueOnly);
+  }
+  if (!value.isEmpty()) {
+    header->setValue("Cookie", value.join("; "));
+    DLOG("Cookie:%s", value.join("; ").toStdString().c_str());
+  }
+}
+
+static void SaveCookie(const QUrl& url, const QHttpResponseHeader& header) {
+  QStringList list = header.allValues("Set-Cookie");
+  if (list.size() != 0) DLOG("Get Cookie Line: %d", list.size());
+  foreach (QString item, list) {
+    QList<QNetworkCookie> cookies =
+        QNetworkCookie::parseCookies(item.toAscii());
+    cookie_jar->setCookiesFromUrl(cookies, url);
+  }
+}
+#else
+static void RestoreCookie(const QUrl& url, QHttpRequestHeader *header);
+static void SaveCookie(const QUrl& url, const QHttpResponseHeader& header);
+#endif
+
+class XMLHttpRequest : public QObject,
+                       public ScriptableHelper<XMLHttpRequestInterface> {
  public:
   DEFINE_CLASS_ID(0xa34d00e04d0acfbb, XMLHttpRequestInterface);
 
-  XMLHttpRequest(Session *session, MainLoopInterface *main_loop,
+  XMLHttpRequest(QObject *parent, MainLoopInterface *main_loop,
                  XMLParserInterface *xml_parser,
                  const QString &default_user_agent)
-      : main_loop_(main_loop),
+      : QObject(parent),
+        main_loop_(main_loop),
         xml_parser_(xml_parser),
         default_user_agent_(default_user_agent),
         http_(NULL),
         request_header_(NULL),
-        session_(session),
-        handler_(NULL),
         send_data_(NULL),
         async_(false),
+        no_cookie_(false),
         state_(UNSENT),
         send_flag_(false),
         redirected_times_(0),
@@ -198,11 +204,9 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
 
     url_ = url;
     host_ = qurl.host().toStdString();
-    delete http_;
-    http_ = new QHttp(qurl.host(), mode);
+    if (http_) http_->deleteLater();
+    http_ = new MyHttp(qurl.host(), mode, this);
     http_->setUser(user_, password_);
-    delete handler_;
-    handler_ = new HttpHandler(this, http_);
 
     std::string path = "/";
     size_t sep = url_.find('/', qurl.scheme().length() + strlen("://"));
@@ -272,6 +276,14 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
       return NO_ERR;
     }
 
+    if (strcasecmp(header, "Cookie") == 0 &&
+        value && strcasecmp(value, "none") == 0) {
+      // Microsoft XHR hidden feature: setRequestHeader('Cookie', 'none')
+      // clears all cookies. Some gadgets (e.g. reader) use this.
+      no_cookie_ = true;
+      return NO_ERR;
+    }
+
     if (IsUniqueHeader(header))
       request_header_->setValue(header, value);
     else
@@ -297,9 +309,6 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
 
     send_flag_ = true;
     if (async_) {
-      // Add an internal reference when this request is working to prevent
-      // this object from being GC'ed.
-      Ref();
       // Do backoff checking to avoid DDOS attack to the server.
       if (!IsXHRBackoffRequestOK(main_loop_->GetCurrentTime(),
                                  host_.c_str())) {
@@ -308,8 +317,11 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
         // this kind of exception.
         return NO_ERR;
       }
-      if (session_)
-        session_->RestoreCookie(request_header_);
+      // Add an internal reference when this request is working to prevent
+      // this object from being GC'ed.
+      Ref();
+      if (!no_cookie_)
+        RestoreCookie(QUrl(url_.c_str()), request_header_);
 
       if (data.size()) {
         send_data_ = new QByteArray(data.c_str(),
@@ -369,13 +381,11 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
   }
 
   void FreeResource() {
-    delete handler_;
-    handler_ = NULL;
     delete request_header_;
     request_header_ = NULL;
     delete send_data_;
     send_data_ = NULL;
-    delete http_;
+    if (http_) http_->deleteLater();
     http_ = NULL;
 
     response_headers_.clear();
@@ -540,7 +550,7 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
   }
 
   virtual Connection *ConnectOnDataReceived(
-      Slot1<size_t, const std::string &> *receiver) {
+      Slot2<size_t, const void *, size_t> *receiver) {
     return ondatareceived_signal_.Connect(receiver);
   }
 
@@ -655,8 +665,7 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
       if (ChangeState(HEADERS_RECEIVED))
         ChangeState(LOADING);
     }
-    if (session_)
-      session_->SaveCookie(header);
+    SaveCookie(QUrl(url_.c_str()), header);
   }
 
   void OnRequestFinished(int id, bool error) {
@@ -695,8 +704,9 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
       Done(false, false);
     } else {
       redirected_times_++;
-      if (session_)
-        session_->RestoreCookie(request_header_);
+      // FIXME(idlecat): What is the right behavior when redirected?
+      if (no_cookie_)
+        RestoreCookie(QUrl(url_.c_str()), request_header_);
 
       if (send_data_)
         http_->request(*request_header_, *send_data_);
@@ -708,17 +718,16 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
   MainLoopInterface *main_loop_;
   XMLParserInterface *xml_parser_;
   QString default_user_agent_;
-  QHttp *http_;
+  MyHttp *http_;
   QHttpRequestHeader *request_header_;
   QHttpResponseHeader response_header_;
-  Session *session_;
-  HttpHandler *handler_;
   QByteArray *send_data_;
   Signal0<void> onreadystatechange_signal_;
-  Signal1<size_t, const std::string &> ondatareceived_signal_;
+  Signal2<size_t, const void *, size_t> ondatareceived_signal_;
 
   std::string url_, host_;
   bool async_;
+  bool no_cookie_;
 
   State state_;
   // Required by the specification.
@@ -742,29 +751,36 @@ class XMLHttpRequest : public ScriptableHelper<XMLHttpRequestInterface> {
   CaseInsensitiveStringMap response_headers_map_;
 };
 
-void HttpHandler::OnResponseHeaderReceived(const QHttpResponseHeader& header) {
+void MyHttp::OnResponseHeaderReceived(const QHttpResponseHeader& header) {
   request_->OnResponseHeaderReceived(header);
 }
 
-void HttpHandler::OnDone(bool error) {
+void MyHttp::OnDone(bool error) {
   request_->OnRequestFinished(0, error);
 }
 
+static size_t kMaxSessionNumber = 100000;   // An arbitary number
 class XMLHttpRequestFactory : public XMLHttpRequestFactoryInterface {
  public:
   XMLHttpRequestFactory() : next_session_id_(1) {
   }
 
   virtual int CreateSession() {
-    int result = next_session_id_++;
-    sessions_[result] = new Session();
-    return result;
+    if (kMaxSessionNumber < sessions_.size()) return -1;
+    while (1) {
+      int result = next_session_id_++;
+      if (result < 0) result = next_session_id_ = 1;
+      if (sessions_.find(result) == sessions_.end()) {
+        sessions_[result] = new QObject();
+        return result;
+      }
+    }
   }
 
   virtual void DestroySession(int session_id) {
     Sessions::iterator it = sessions_.find(session_id);
     if (it != sessions_.end()) {
-      Session *session = it->second;
+      QObject *session = it->second;
       delete session;
       sessions_.erase(it);
     } else {
@@ -795,7 +811,7 @@ class XMLHttpRequestFactory : public XMLHttpRequestFactoryInterface {
   }
 
  private:
-  typedef std::map<int, Session*> Sessions;
+  typedef LightMap<int, QObject*> Sessions;
   Sessions sessions_;
   int next_session_id_;
   QString default_user_agent_;
@@ -809,9 +825,48 @@ class XMLHttpRequestFactory : public XMLHttpRequestFactoryInterface {
 
 static ggadget::qt::XMLHttpRequestFactory gFactory;
 
+// parse proxy env value, which may look like:
+// "http://username:password@yourproxy.com:8080"
+static bool ParseProxyEnv(const QString &value, QString *host, quint16 *port,
+                          QString *user, QString *password) {
+  QRegExp re("(^.*://)?((.+)(:(.+))?@)?([^:]+)(:([0-9]+))?");
+
+  if (re.indexIn(value) == -1) return false;
+  *host = re.cap(6);
+  if (re.cap(8) != "") {
+    *port = static_cast<quint16>(re.cap(8).toInt());
+  } else {
+    *port = 80;
+  }
+  *user = re.cap(3);
+  *password = re.cap(5);
+  return true;
+}
+
 extern "C" {
   bool Initialize() {
     LOGI("Initialize qt_xml_http_request extension.");
+    const char *proxy_names[] = {
+      "all_proxy", "http_proxy", "https_proxy", NULL
+    };
+
+    QString host, user, password;
+    quint16 port = 0;
+    for (int i = 0; proxy_names[i]; i++) {
+      const char *env = getenv(proxy_names[i]);
+      if (env && ParseProxyEnv(env, &host, &port, &user, &password)) {
+        QNetworkProxy proxy;
+        proxy.setType(QNetworkProxy::HttpProxy);
+        proxy.setHostName(host);
+        proxy.setPort(port);
+        if (user != "") proxy.setUser(user);
+        if (password != "") proxy.setPassword(password);
+        QNetworkProxy::setApplicationProxy(proxy);
+        DLOG("Using proxy %s:%d", host.toUtf8().data(), port);
+        break;
+      }
+    }
+    ggadget::qt::cookie_jar = new QNetworkCookieJar;
     return ggadget::SetXMLHttpRequestFactory(&gFactory);
   }
 
