@@ -38,8 +38,6 @@
 namespace ggadget {
 namespace smjs {
 
-// The maximum execution time of a piece of script (10 seconds).
-static const uint64_t kMaxScriptRunTime = 10000;
 // OperationCallback is checked after the script executed instructions
 // weighted 100 * JS_OPERATION_WEIGHT_BASE (new version), or 100 branches
 // (old version, deperecated).
@@ -159,12 +157,17 @@ JSScriptContext::JSScriptContext(JSScriptRuntime *runtime, JSContext *context)
       lineno_(0) {
   JS_SetContextPrivate(context_, this);
   JS_SetLocaleCallbacks(context_, &gLocaleCallbacks);
+#ifdef HAVE_JS_SetOperationCallback
 #ifdef JS_OPERATION_WEIGHT_BASE
   JS_SetOperationCallback(context_, OperationCallback,
                           kOperationCallbackWeight);
 #else
-  JS_SetBranchCallback(context_, BranchCallback);
+  // This func in newer SpiderMonkey in xulrunner 1.9.1 has a different
+  // prototype.
+  JS_SetOperationCallback(context_, OperationCallback);
 #endif
+#endif
+
   JS_SetErrorReporter(context, ReportError);
   // JS_SetOptions(context_, JS_GetOptions(context_) | JSOPTION_STRICT);
 
@@ -457,9 +460,6 @@ bool JSScriptContext::SetGlobalObject(ScriptableInterface *global_object) {
 
 JSBool JSScriptContext::ConstructObject(JSContext *cx, JSObject *obj,
                                         uintN argc, jsval *argv, jsval *rval) {
-  AutoLocalRootScope local_root_scope(cx);
-  if (!local_root_scope.good())
-    return JS_FALSE;
   JSScriptContext *context_wrapper = GetJSScriptContext(cx);
   ASSERT(context_wrapper);
   ScopedLogContext log_context(context_wrapper);
@@ -683,17 +683,6 @@ JSBool JSScriptContext::OperationCallback(JSContext *cx) {
   return JS_TRUE;
 }
 
-#ifndef JS_OPERATION_WEIGHT_BASE
-JSBool JSScriptContext::BranchCallback(JSContext *cx, JSScript *script) {
-  static uint32 count = 0;
-  if (++count == kOperationCallbackMultiply) {
-    count = 0;
-    return OperationCallback(cx);
-  }
-  return JS_TRUE;
-}
-#endif
-
 bool JSScriptContext::OnClearOperationTimeTimer(int watch_id) {
   ASSERT(watch_id == reset_operation_time_timer_);
   operation_callback_time_ = 0;
