@@ -102,7 +102,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetPixelWidth(double width) {
     if (width >= 0.0 && (width != width_ || width_relative_)) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       width_ = width;
       width_relative_ = false;
       WidthChanged();
@@ -111,7 +111,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetPixelHeight(double height) {
     if (height >= 0.0 && (height != height_ || height_relative_)) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       height_ = height;
       height_relative_ = false;
       HeightChanged();
@@ -120,7 +120,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetRelativeWidth(double width) {
     if (width >= 0.0 && (width != pwidth_ || !width_relative_)) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       pwidth_ = width;
       width_relative_ = true;
       WidthChanged();
@@ -129,7 +129,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetRelativeHeight(double height) {
     if (height >= 0.0 && (height != pheight_ || !height_relative_)) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       pheight_ = height;
       height_relative_ = true;
       HeightChanged();
@@ -138,7 +138,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetPixelX(double x) {
     if (x != x_ || x_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       x_ = x;
       x_relative_ = false;
       PositionChanged();
@@ -147,7 +147,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetPixelY(double y) {
     if (y != y_ || y_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       y_ = y;
       y_relative_ = false;
       PositionChanged();
@@ -156,7 +156,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetRelativeX(double x) {
     if (x != px_ || !x_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       px_ = x;
       x_relative_ = true;
       PositionChanged();
@@ -165,7 +165,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetRelativeY(double y) {
     if (y != py_ || !y_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       py_ = y;
       y_relative_ = true;
       PositionChanged();
@@ -174,7 +174,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetPixelPinX(double pin_x) {
     if (pin_x != pin_x_ || pin_x_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       pin_x_ = pin_x;
       pin_x_relative_ = false;
       PositionChanged();
@@ -183,7 +183,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetPixelPinY(double pin_y) {
     if (pin_y != pin_y_ || pin_y_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       pin_y_ = pin_y;
       pin_y_relative_ = false;
       PositionChanged();
@@ -192,7 +192,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetRelativePinX(double pin_x) {
     if (pin_x != ppin_x_ || !pin_x_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       ppin_x_ = pin_x;
       pin_x_relative_ = true;
       PositionChanged();
@@ -201,7 +201,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetRelativePinY(double pin_y) {
     if (pin_y != ppin_y_ || !pin_y_relative_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       ppin_y_ = pin_y;
       pin_y_relative_ = true;
       PositionChanged();
@@ -210,7 +210,7 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetRotation(double rotation) {
     if (rotation != rotation_) {
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       rotation_ = rotation;
       PositionChanged();
     }
@@ -218,6 +218,9 @@ class BasicElement::Impl : public SmallObject<> {
 
   void SetOpacity(double opacity) {
     if (opacity != opacity_) {
+      // Assumes the visibility is changed if opacity is changed from/to 0.
+      if (opacity == 0 || opacity_ == 0)
+        visibility_changed_ = true;
       opacity_ = opacity;
       QueueDraw();
     }
@@ -491,9 +494,9 @@ class BasicElement::Impl : public SmallObject<> {
       ppin_y_ = height_ > 0.0 ? pin_y_ / height_ : 0.0;
     }
 
-    if ((position_changed_ || size_changed_ || visibility_changed_) &&
-        IsReallyVisible())
-      view_->AddElementToClipRegion(owner_, NULL);
+    if (position_changed_ || size_changed_ || visibility_changed_) {
+      AddToClipRegion(NULL);
+    }
 
     if (size_changed_)
       PostSizeEvent();
@@ -501,14 +504,8 @@ class BasicElement::Impl : public SmallObject<> {
       children_->Layout();
 
     if (content_changed_) {
-      if (cache_enabled_) {
-        // Add it into clip region, so that the canvas cache can be updated
-        // correctly.
-        view_->AddElementToClipRegion(owner_, NULL);
-      }
       // To let all associated copy elements to update their content.
       FireOnContentChangedSignal();
-      content_changed_ = false;
     }
 
     // No other code will use it before calling Draw(), so it's safe to reset.
@@ -553,11 +550,11 @@ class BasicElement::Impl : public SmallObject<> {
       // - Opacity of the element is not 1.0 and it has children.
       // - Canvas cache is enabled.
       bool indirect_draw = cache_enabled_ || mask || flip_ != FLIP_NONE ||
-                          (opacity_ != 1.0 && children_ &&
-                           children_->GetCount());
+          (opacity_ != 1.0 && children_ && children_->GetCount());
       if (indirect_draw) {
         if (cache_) {
           target = cache_;
+          target->PushState();
         } else {
           target = view_->GetGraphics()->NewCanvas(width, height);
           force_draw = true;
@@ -600,6 +597,8 @@ class BasicElement::Impl : public SmallObject<> {
         // Don't destroy canvas cache.
         if (cache_ != target)
           target->Destroy();
+        else
+          target->PopState();
       }
 
       canvas->PopState();
@@ -647,10 +646,54 @@ class BasicElement::Impl : public SmallObject<> {
 #endif
 
  public:
+  void AddToClipRegion(const Rectangle *rect) {
+    if ((visible_ && opacity_ != 0.0) || visibility_changed_) {
+      if (rect) {
+        clip_region_.AddRectangle(owner_->GetRectExtentsInView(*rect));
+      } else {
+        clip_region_.AddRectangle(owner_->GetExtentsInView());
+      }
+    }
+  }
+
+  void AggregateClipRegion(const Rectangle &boundary, ClipRegion *region) {
+    if (region != NULL && !boundary.IsEmpty()) {
+      size_t count = clip_region_.GetRectangleCount();
+      for (size_t i = 0; i < count; ++i) {
+        Rectangle rect = clip_region_.GetRectangle(i);
+        if (rect.Intersect(boundary)) {
+          rect.Integerize(true);
+          region->AddRectangle(rect);
+        }
+      }
+
+      if (visible_ && opacity_ != 0.0) {
+        Rectangle extents = owner_->GetExtentsInView();
+        if (extents.Intersect(boundary)) {
+          if (children_ && children_->GetCount()) {
+            children_->AggregateClipRegion(extents, region);
+          }
+          owner_->AggregateMoreClipRegion(extents, region);
+          // Short circuit.
+          clip_region_.Clear();
+          return;
+        }
+      }
+    }
+
+    clip_region_.Clear();
+    // children's AggregateClipRegion() and AggregateMoreClipRegion() must be
+    // called in order to let children clear their clip region cache.
+    if (children_ && children_->GetCount()) {
+      children_->AggregateClipRegion(Rectangle(), NULL);
+    }
+    owner_->AggregateMoreClipRegion(Rectangle(), NULL);
+  }
+
   void QueueDraw() {
     if ((visible_ || visibility_changed_) && !draw_queued_) {
       draw_queued_ = true;
-      view_->AddElementToClipRegion(owner_, NULL);
+      AddToClipRegion(NULL);
       view_->QueueDraw();
       if (!content_changed_)
         MarkContentChanged();
@@ -664,7 +707,7 @@ class BasicElement::Impl : public SmallObject<> {
     if ((visible_ || visibility_changed_) && !draw_queued_) {
       // Don't set draw_queued_, because it's only queued part of the element.
       // Other part might be queued later.
-      view_->AddElementToClipRegion(owner_, &rect);
+      AddToClipRegion(&rect);
       view_->QueueDraw();
       if (!content_changed_)
         MarkContentChanged();
@@ -674,22 +717,18 @@ class BasicElement::Impl : public SmallObject<> {
 #endif
   }
 
-  bool QueueClipRectCallback(double x, double y, double w, double h) {
-    Rectangle rect(x, y, w, h);
-    view_->AddElementToClipRegion(owner_, &rect);
-    return true;
-  }
-
   void QueueDrawRegion(const ClipRegion &region) {
     if ((visible_ || visibility_changed_) && !draw_queued_) {
       // Don't set draw_queued_, because it's only queued part of the element.
       // Other part might be queued later.
-      if (region.EnumerateRectangles(
-          NewSlot(this, &Impl::QueueClipRectCallback))) {
-        view_->QueueDraw();
-        if (!content_changed_)
-          MarkContentChanged();
+      size_t count = region.GetRectangleCount();
+      for (size_t i = 0; i < count; ++i) {
+        Rectangle rect = region.GetRectangle(i);
+        AddToClipRegion(&rect);
       }
+      view_->QueueDraw();
+      if (!content_changed_)
+        MarkContentChanged();
     }
 #ifdef _DEBUG
     ++total_queue_draw_count_;
@@ -706,13 +745,6 @@ class BasicElement::Impl : public SmallObject<> {
   void FireOnContentChangedSignal() {
     if (on_content_changed_signal_.HasActiveConnections())
       on_content_changed_signal_();
-
-    // Fire content changed signal of its parents.
-    BasicElement *elm = owner_->GetParentElement();
-    for (; elm; elm = elm->GetParentElement()) {
-      if (elm->impl_->on_content_changed_signal_.HasActiveConnections())
-        elm->impl_->on_content_changed_signal_();
-    }
   }
 
   void PostSizeEvent() {
@@ -752,10 +784,14 @@ class BasicElement::Impl : public SmallObject<> {
     QueueDraw();
   }
 
-  bool IsReallyVisible() {
-    return visible_ && opacity_ != 0.0 &&
+  // clip: whether the visible status effected by clipping. If clip is true
+  // and the element is out of parent's visible area, this method returns
+  // false.
+  bool IsReallyVisible(bool clip) {
+    return visible_ && opacity_ != 0.0 && width_ > 0 && height_ > 0 &&
         (!parent_ ||
-         (parent_->IsReallyVisible() && parent_->IsChildInVisibleArea(owner_)));
+         (parent_->impl_->IsReallyVisible(clip) &&
+          (!clip || parent_->IsChildInVisibleArea(owner_))));
   }
 
   ViewInterface::HitTest GetHitTest() {
@@ -973,9 +1009,6 @@ class BasicElement::Impl : public SmallObject<> {
   }
 
   EventResult OnOtherEvent(const Event &event) {
-    if (!enabled_)
-      return EVENT_RESULT_UNHANDLED;
-
     ElementHolder this_element_holder(owner_);
     ScriptableEvent scriptable_event(&event, owner_, NULL);
 #if defined(_DEBUG) && defined(EVENT_VERBOSE_DEBUG)
@@ -983,9 +1016,18 @@ class BasicElement::Impl : public SmallObject<> {
          name_.c_str(), tag_name_);
 #endif
 
-    // TODO: focus logic.
     switch (event.GetType()) {
       case Event::EVENT_FOCUS_IN:
+        if (!enabled_) return EVENT_RESULT_UNHANDLED;
+        if (parent_) {
+          double left, top, right, bottom;
+          GetChildRectExtentInParent(x_, y_, pin_x_, pin_y_,
+                                     DegreesToRadians(rotation_),
+                                     0, 0, width_, height_,
+                                     &left, &top, &right, &bottom);
+          parent_->EnsureAreaVisible(
+              Rectangle(left, top, right - left, bottom - top), owner_);
+        }
         view_->FireEvent(&scriptable_event, onfocusin_event_);
         break;
       case Event::EVENT_FOCUS_OUT:
@@ -1019,6 +1061,8 @@ class BasicElement::Impl : public SmallObject<> {
   std::string name_;
   std::string tooltip_;
 
+  ClipRegion clip_region_;
+
   EventSignal onclick_event_;
   EventSignal ondblclick_event_;
   EventSignal onrclick_event_;
@@ -1049,7 +1093,7 @@ class BasicElement::Impl : public SmallObject<> {
   static int total_draw_count_;
   static int total_queue_draw_count_;
 
-  static std::map<uint64_t, bool> class_has_children_;
+  static LightMap<uint64_t, bool> class_has_children_;
 #endif
 
   ViewInterface::HitTest hittest_   : 6;
@@ -1082,7 +1126,7 @@ class BasicElement::Impl : public SmallObject<> {
 int BasicElement::Impl::total_debug_color_index_ = 0;
 int BasicElement::Impl::total_draw_count_ = 0;
 int BasicElement::Impl::total_queue_draw_count_ = 0;
-std::map<uint64_t, bool> BasicElement::Impl::class_has_children_;
+LightMap<uint64_t, bool> BasicElement::Impl::class_has_children_;
 #endif
 
 // Must sync with ViewInterface::CursorType enumerates
@@ -1618,11 +1662,11 @@ void BasicElement::SetVisible(bool visible) {
 }
 
 bool BasicElement::IsReallyVisible() const {
-  return impl_->IsReallyVisible();
+  return impl_->IsReallyVisible(true);
 }
 
 bool BasicElement::IsReallyEnabled() const {
-  return impl_->enabled_ && IsReallyVisible();
+  return impl_->enabled_ && impl_->IsReallyVisible(false);
 }
 
 bool BasicElement::IsFullyOpaque() const {
@@ -2001,7 +2045,15 @@ bool BasicElement::OnAddContextMenuItems(MenuInterface *menu) {
 }
 
 bool BasicElement::IsChildInVisibleArea(const BasicElement *child) const {
-  return true;
+  double min_x, min_y, max_x, max_y;
+  GetChildRectExtentInParent(child->GetPixelX(), child->GetPixelY(),
+                             child->GetPixelPinX(), child->GetPixelPinY(),
+                             DegreesToRadians(child->GetRotation()),
+                             0, 0,
+                             child->GetPixelWidth(), child->GetPixelHeight(),
+                             &min_x, &min_y, &max_x, &max_y);
+  return max_x > 0 && max_y > 0 &&
+         min_x < GetPixelWidth() && min_y < GetPixelHeight();
 }
 
 bool BasicElement::HasOpaqueBackground() const {
@@ -2098,4 +2150,30 @@ EventResult BasicElement::HandleOtherEvent(const Event &event) {
   return EVENT_RESULT_UNHANDLED;
 }
 
+void BasicElement::AggregateClipRegion(const Rectangle &boundary,
+                                       ClipRegion *region) {
+  impl_->AggregateClipRegion(boundary, region);
+}
+
+void BasicElement::AggregateMoreClipRegion(const Rectangle &boundary,
+                                           ClipRegion *region) {
+}
+
+void BasicElement::EnsureAreaVisible(const Rectangle &rect,
+                                     const BasicElement *source) {
+  // First test if the rectangle is visible in this element.
+  if (impl_->parent_ && rect.x + rect.w > 0 && rect.y + rect.h > 0 &&
+      rect.x < GetPixelWidth() && rect.y < GetPixelHeight()) {
+    double left, top, right, bottom;
+    GetChildRectExtentInParent(GetPixelX(), GetPixelY(),
+                               GetPixelPinX(), GetPixelPinY(),
+                               DegreesToRadians(GetRotation()),
+                               rect.x, rect.y,
+                               rect.x + rect.w, rect.y + rect.h,
+                               &left, &top, &right, &bottom);
+    impl_->parent_->EnsureAreaVisible(
+        Rectangle(left, top, right - left, bottom - top), this);
+  }
+}
+
 } // namespace ggadget
