@@ -528,6 +528,9 @@ class Gadget::Impl : public ScriptableHelperNativeOwnedDefault {
 
   void RemoveMe(bool save_data) {
     if (!remove_me_timer_) {
+      if (!save_data) {
+        options_->DeleteStorage();
+      }
       remove_me_timer_ = GetGlobalMainLoop()->AddTimeoutWatch(
           0, new RemoveMeWatchCallback(host_, owner_, save_data));
     }
@@ -543,6 +546,10 @@ class Gadget::Impl : public ScriptableHelperNativeOwnedDefault {
     host_->ShowGadgetDebugConsole(owner_);
   }
 
+  void FeedbackMenuCallback(const char *, const std::string &url) {
+    host_->OpenURL(NULL, url.c_str());
+  }
+
   void OnAddCustomMenuItems(MenuInterface *menu) {
     ScriptableMenu *smenu = new ScriptableMenu(owner_, menu);
     smenu->Ref();
@@ -554,6 +561,15 @@ class Gadget::Impl : public ScriptableHelperNativeOwnedDefault {
                     NewSlot(this, &Impl::DebugConsoleMenuCallback),
                     MenuInterface::MENU_ITEM_PRI_GADGET);
     }
+
+    if (ongetfeedbackurl_signal_.HasActiveConnections()) {
+      std::string url = ongetfeedbackurl_signal_();
+      if (url.length()) {
+        menu->AddItem(GM_("MENU_ITEM_FEEDBACK"), 0, 0,
+                      NewSlot(this, &Impl::FeedbackMenuCallback, url),
+                      MenuInterface::MENU_ITEM_PRI_GADGET);
+      }
+    }
     // Remove item is added in view decorator.
   }
 
@@ -1009,7 +1025,7 @@ class Gadget::Impl : public ScriptableHelperNativeOwnedDefault {
   Signal1<void, int> onpluginflagschanged_signal_;
   Signal2<void, LogLevel, const std::string &> log_signal_;
 
-  Signal0<void> onopenfeedbackurl_signal_;
+  Signal0<std::string> ongetfeedbackurl_signal_;
 
   StringMap manifest_info_map_;
   StringMap strings_map_;
@@ -1179,8 +1195,8 @@ Connection *Gadget::ConnectOnPluginFlagsChanged(Slot1<void, int> *handler) {
   return impl_->onpluginflagschanged_signal_.Connect(handler);
 }
 
-Connection *Gadget::ConnectOnOpenFeedbackURL(Slot0<void> *handler) {
-  return impl_->onopenfeedbackurl_signal_.Connect(handler);
+Connection *Gadget::ConnectOnGetFeedbackURL(Slot0<std::string> *handler) {
+  return impl_->ongetfeedbackurl_signal_.Connect(handler);
 }
 
 XMLHttpRequestInterface *Gadget::CreateXMLHttpRequest() {
@@ -1263,7 +1279,7 @@ void Gadget::ShowAboutDialog() {
   if (ContainsHTML(about.c_str()))
     about = ExtractTextFromHTML(about.c_str());
 
-  std::map<std::string, Variant> params;
+  LightMap<std::string, Variant> params;
   params["title"] = Variant(title);
   params["copyright"] = Variant(copyright);
   params["about"] = Variant(about);
@@ -1281,14 +1297,6 @@ void Gadget::ShowAboutDialog() {
                        NewScriptableMap(params));
 }
 
-bool Gadget::HasFeedbackURL() const {
-  return impl_->onopenfeedbackurl_signal_.HasActiveConnections();
-}
-
-void Gadget::OpenFeedbackURL() {
-  impl_->onopenfeedbackurl_signal_();
-}
-
 // static methods
 bool Gadget::GetGadgetManifest(const char *base_path, StringMap *data) {
   return GetGadgetManifestForLocale(base_path, NULL, data);
@@ -1360,14 +1368,11 @@ bool Gadget::SaveGadgetInitialPermissions(const char *options_path,
   OptionsInterface *options = CreateOptions(options_path);
   ASSERT(options);
   if (options) {
-    Variant value = options->GetInternalValue(kPermissionsOption);
-    if (value.type() != Variant::TYPE_STRING) {
-      Permissions tmp = permissions;
-      tmp.RemoveAllRequired();
-      options->PutInternalValue(kPermissionsOption, Variant(tmp.ToString()));
-      options->Flush();
-      result = true;
-    }
+    Permissions tmp = permissions;
+    tmp.RemoveAllRequired();
+    options->PutInternalValue(kPermissionsOption, Variant(tmp.ToString()));
+    options->Flush();
+    result = true;
     delete options;
   }
   return result;
