@@ -20,6 +20,8 @@
 #include <ctime>
 #include <string>
 #include <map>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include "dir_file_manager.h"
 #include "element_factory.h"
@@ -220,7 +222,7 @@ static bool DialogCallback(int flag, View *view) {
 }
 
 bool ShowDialogView(HostInterface *host, const char *location, int flags,
-                    const std::map<std::string, Variant> &params) {
+                    const LightMap<std::string, Variant> &params) {
   FileManagerInterface *file_manager = GetGlobalFileManager();
   std::string xml;
   if (!file_manager || !file_manager->ReadFile(location, &xml))
@@ -255,10 +257,14 @@ static void AboutOpenURL(const char *url, HostInterface *host) {
 }
 
 void ShowAboutDialog(HostInterface *host) {
-  std::map<std::string, Variant> params;
+  LightMap<std::string, Variant> params;
   params["title"] = Variant(GMS_("GOOGLE_GADGETS"));
   params["version"] = Variant(std::string(GGL_VERSION) + " " +
-      StringPrintf(GM_("API_VERSION"), GGL_API_VERSION));
+      StringPrintf(GM_("API_VERSION"), GGL_API_VERSION)
+#ifdef _DEBUG
+      + StringPrintf("\npid: %u", getpid())
+#endif
+      );
   params["copyright"] = Variant(GMS_("GGL_COPYRIGHT"));
   params["description"] = Variant(GMS_("GGL_DESCRIPTION"));
   // AnchorElement's OpenURL() doesn't work because the view is run without
@@ -349,7 +355,7 @@ class HostArgumentParser::Impl {
   bool started_;
   bool error_occurred_;
 
-  typedef std::map<int, Variant> ArgumentValueMap;
+  typedef LightMap<int, Variant> ArgumentValueMap;
   ArgumentValueMap specified_args_;
   StringVector remained_args_;
 };
@@ -562,34 +568,36 @@ bool HostArgumentParser::EnumerateRemainedArgs(
 const char HostArgumentParser::kStartSignature[] = "<|START|>";
 const char HostArgumentParser::kFinishSignature[] = "<|FINISH|>";
 
-void SetupGadgetOpenFeedbackURLHandler(Gadget *gadget) {
-  class OpenFeedbackURLSlot : public Slot0<void> {
+void SetupGadgetGetFeedbackURLHandler(Gadget *gadget) {
+  class GetFeedbackURLSlot : public Slot0<std::string> {
    public:
-    OpenFeedbackURLSlot(Gadget *gadget, const std::string &url)
-      : gadget_(gadget), url_(url) {
+    GetFeedbackURLSlot(Gadget *gadget)
+      : gadget_(gadget), url_retrieved_(false) {
     }
     virtual ResultVariant Call(ScriptableInterface *object,
                                int argc, const Variant argv[]) const {
-      bool old_in_user_interaction = gadget_->SetInUserInteraction(true);
-      gadget_->OpenURL(url_.c_str());
-      gadget_->SetInUserInteraction(old_in_user_interaction);
-      return ResultVariant();
+      if (!url_retrieved_) {
+        GadgetManagerInterface *gadget_manager = GetGadgetManager();
+        if (gadget_ && gadget_manager) {
+          url_ = gadget_manager->GetGadgetInstanceFeedbackURL(
+              gadget_->GetInstanceID());
+        }
+        url_retrieved_ = true;
+      }
+
+      return ResultVariant(Variant(url_));
     }
     virtual bool operator==(const Slot &another) const {
       return false;
     }
    private:
     Gadget *gadget_;
-    std::string url_;
+    mutable std::string url_;
+    mutable bool url_retrieved_;
   };
 
-  GadgetManagerInterface *gadget_manager = GetGadgetManager();
-  if (gadget && gadget_manager) {
-    std::string url =
-        gadget_manager->GetGadgetInstanceFeedbackURL(gadget->GetInstanceID());
-    if (url.length()) {
-      gadget->ConnectOnOpenFeedbackURL(new OpenFeedbackURLSlot(gadget, url));
-    }
+  if (gadget) {
+    gadget->ConnectOnGetFeedbackURL(new GetFeedbackURLSlot(gadget));
   }
 }
 
