@@ -800,8 +800,11 @@ class SideBarGtkHost::Impl {
                           reinterpret_cast<guchar *>(&struts), 12);
     } else {
       has_strut_ = false;
-      gdk_property_delete(sidebar_window_->window, net_wm_strut_);
-      gdk_property_delete(sidebar_window_->window, net_wm_strut_partial_);
+      if (net_wm_strut_ != GDK_NONE)
+        gdk_property_delete(sidebar_window_->window, net_wm_strut_);
+      if (net_wm_strut_partial_ != GDK_NONE)
+        gdk_property_delete(sidebar_window_->window, net_wm_strut_partial_);
+
       sidebar_host_->SetWindowType((flags_ & MATCHBOX_WORKAROUND) ?
                                    GDK_WINDOW_TYPE_HINT_DIALOG :
                                    GDK_WINDOW_TYPE_HINT_NORMAL);
@@ -1327,7 +1330,7 @@ class SideBarGtkHost::Impl {
       return NULL;
     }
 
-    SetupGadgetOpenFeedbackURLHandler(gadget);
+    SetupGadgetGetFeedbackURLHandler(gadget);
 
     if (gadget->GetDisplayTarget() == Gadget::TARGET_SIDEBAR) {
       MainViewDecoratorBase *view_decorator =
@@ -1572,8 +1575,6 @@ class SideBarGtkHost::Impl {
 
   void RemoveGadget(Gadget *gadget, bool save_data) {
     ASSERT(gadget);
-    CloseDetailsView(gadget->GetInstanceID());
-    OnMainViewPopIn(gadget->GetInstanceID());
     int id = gadget->GetInstanceID();
     // If RemoveGadgetInstance() returns false, then means this instance is not
     // installed by gadget manager.
@@ -1581,9 +1582,20 @@ class SideBarGtkHost::Impl {
       RemoveGadgetInstanceCallback(id);
   }
 
+  void SaveGadgetDisplayTargetInfo(const GadgetInfo &info) {
+    OptionsInterface *opt = info.gadget->GetOptions();
+    opt->PutInternalValue(kOptionDisplayTarget,
+                          Variant(info.gadget->GetDisplayTarget()));
+    opt->PutInternalValue(kOptionPositionInSideBar,
+                          Variant(info.index_in_sidebar));
+  }
+
   void RemoveGadgetInstanceCallback(int instance_id) {
     GadgetInfoMap::iterator it = gadgets_.find(instance_id);
     if (it != gadgets_.end()) {
+      CloseDetailsView(instance_id);
+      OnMainViewPopIn(instance_id);
+      SaveGadgetDisplayTargetInfo(it->second);
       if (it->second.debug_console)
         gtk_widget_destroy(it->second.debug_console);
       delete it->second.gadget;
@@ -1889,7 +1901,7 @@ class SideBarGtkHost::Impl {
  public:  // members
   GadgetBrowserHost gadget_browser_host_;
 
-  typedef std::map<int, GadgetInfo> GadgetInfoMap;
+  typedef LightMap<int, GadgetInfo> GadgetInfoMap;
   GadgetInfoMap gadgets_;
 
   SideBarGtkHost *owner_;
